
#include "TheClock.h"

void DrawPointer(BView* target, BPoint center, float angle, float factor, float backFactor, float widthFactor) {
	BPolygon* ptr=new BPolygon();
	ptr->AddPoints(&BPoint(center.x+sin(angle)*factor*center.x, center.y-cos(angle)*factor*center.y),1);
	ptr->AddPoints(&BPoint(center.x+sin(angle+PI/2)*widthFactor*center.x, center.y-cos(angle+PI/2)*widthFactor*center.y),1);
	ptr->AddPoints(&BPoint(center.x+sin(angle+PI)*backFactor*center.x, center.y-cos(angle+PI)*backFactor*center.y),1);
	ptr->AddPoints(&BPoint(center.x+sin(angle+PI*1.5)*widthFactor*center.y, center.y-cos(angle+PI*1.5)*widthFactor*center.y),1);
	target->FillPolygon(ptr);
	delete ptr;	
};

void DrawClock(BView* target, TheClockView *source) {
	if (source->bgBitmap!=NULL && source->bgType!=BG_NONE) {
		if (source->bgType==BG_PATTERN) {
			for (int y=0; y<target->Bounds().Height(); y+=source->bgBitmap->Bounds().Height())
				for (int x=0; x<target->Bounds().Width(); x+=source->bgBitmap->Bounds().Width())
					target->DrawBitmap(source->bgBitmap,BPoint(x,y)); }
		else {
			target->DrawBitmap(source->bgBitmap,BRect(0,0,source->Bounds().Width(),source->Bounds().Height())); 
		}}
	else {	
		target->SetHighColor(source->bgColor);
		target->FillRect(BRect(0,0,target->Bounds().Width(),target->Bounds().Height()));		
		target->SetHighColor(White);
		target->StrokeLine(BPoint(0,target->Bounds().Height()),BPoint(0,0));
		target->StrokeLine(BPoint(target->Bounds().Width(),0),BPoint(0,0));
		target->SetHighColor(BeShadow);
		target->StrokeLine(BPoint(1,target->Bounds().Height()),BPoint(target->Bounds().Width(),target->Bounds().Height()));
		target->StrokeLine(BPoint(target->Bounds().Width(),1),BPoint(target->Bounds().Width(),target->Bounds().Height()));};
	if (source->digitalClock) {
		target->SetDrawingMode(B_OP_OVER);
		char* timeText=source->timeText;
		char* secondText=source->secondText;
		sprintf(timeText,"%.2d:%.2d",source->timeRecord->tm_hour,source->timeRecord->tm_min);
		sprintf(secondText," %.2d",source->timeRecord->tm_sec);
		target->SetFont(&source->timeFont);
		font_height fh;
		target->GetFontHeight(&fh);	
		int fontHeight=fh.ascent+fh.descent;
		int fontWidth=target->StringWidth("00:00");
		if (source->showSeconds) {
			target->SetFont(&source->secondFont);
			fontWidth+=target->StringWidth(" 00");
			target->SetFont(&source->timeFont);};
		int xoffset=(source->Bounds().Width()-fontWidth)/2;
		int yoffset=source->Bounds().Height()/2+fh.ascent/2-fh.descent/2;
		if (source->castShadow) {
			target->SetHighColor(source->shadowColor);
			target->DrawString(timeText,BPoint(xoffset+3,yoffset+3));
			if (source->showSeconds) {
				target->SetFont(&source->secondFont);
				target->DrawString(secondText);}};
		target->SetHighColor(source->fgColor);
		target->SetFont(&source->timeFont);
		target->DrawString(timeText,BPoint(xoffset,yoffset));
		if (source->showSeconds) {
			target->SetFont(&source->secondFont);
			target->DrawString(secondText);}}
	else { 
		target->SetDrawingMode(B_OP_COPY);
		int center_x;
		int center_y;
		float angle;
		if (source->castShadow) {
			target->SetHighColor(source->shadowColor);
			center_x=source->Bounds().Width()/2;
			center_y=source->Bounds().Height()/2;
			if (source->minuteDots) 
				for (angle=0; angle<2*PI; angle+=PI/30) 
					target->StrokeLine(BPoint(center_x*1.05+sin(angle)*(center_x*0.95), center_y*1.05-cos(angle)*(center_y*0.95)),BPoint(center_x*1.05+sin(angle)*(center_x*0.95), center_y*1.05-cos(angle)*(center_y*0.95)));
			for (angle=0; angle<2*PI; angle+=PI/6)
				target->StrokeLine(BPoint(center_x*1.05+sin(angle)*(center_x*0.87), center_y*1.05-cos(angle)*(center_y*0.87)),BPoint(center_x*1.05+sin(angle)*(center_x*0.95), center_y*1.05-cos(angle)*(center_y*0.95)));
			angle=((source->timeRecord->tm_hour*60+source->timeRecord->tm_min)*PI)/360;
			DrawPointer(target,BPoint(center_x*1.05,center_y*1.05),angle,0.5,0.1,0.05);
			angle=(source->timeRecord->tm_min*12*PI)/360;
			DrawPointer(target,BPoint(center_x*1.05,center_y*1.05),angle,0.8,0.1,0.04);
			if (source->showSeconds) {
				angle=(source->timeRecord->tm_sec*12*PI)/360;
				target->StrokeLine(BPoint(center_x*1.05+sin(angle)*(center_x*0.85), center_y*1.05-cos(angle)*(center_y*0.85)),BPoint(center_x*1.05+sin(angle+PI)*(center_x*0.05), center_y*1.05-cos(angle+PI)*(center_y*0.05)));};
			};
		target->SetHighColor(source->fgColor);
		center_x=source->Bounds().Width()/2;
		center_y=source->Bounds().Height()/2;
		if (source->minuteDots) 
			for (angle=0; angle<2*PI; angle+=PI/30) 
				target->StrokeLine(BPoint(center_x+sin(angle)*(center_x*0.95), center_y-cos(angle)*(center_y*0.95)),BPoint(center_x+sin(angle)*(center_x*0.95), center_y-cos(angle)*(center_y*0.95)));
		for (angle=0; angle<2*PI; angle+=PI/6) 
			target->StrokeLine(BPoint(center_x+sin(angle)*(center_x*0.87), center_y-cos(angle)*(center_y*0.87)),BPoint(center_x+sin(angle)*(center_x*0.95), center_y-cos(angle)*(center_y*0.95)));
		angle=((source->timeRecord->tm_hour*60+source->timeRecord->tm_min)*PI)/360;
		DrawPointer(target,BPoint(center_x,center_y),angle,0.5,0.1,0.05);
		angle=(source->timeRecord->tm_min*12*PI)/360;
		DrawPointer(target,BPoint(center_x,center_y),angle,0.8,0.1,0.04);
		if (source->showSeconds) {
			target->SetHighColor(source->secColor);
			angle=(source->timeRecord->tm_sec*12*PI)/360;
			target->StrokeLine(BPoint(center_x+sin(angle)*(center_x*0.85), center_y-cos(angle)*(center_y*0.85)),BPoint(center_x+sin(angle+PI)*(center_x*0.05), center_y-cos(angle+PI)*(center_y*0.05)));};
		};
};
	
TheClockView::TheClockView( BRect frame, const char* name, uint32 resizingMode, uint32 flags )
	: BView( frame, name, resizingMode, flags )
{
	initialize();	
}

TheClockView::TheClockView(BMessage *msg) 
	:BView(msg) {
	initialize();
	ownWindow=false;
	loadSettings();
	showBordersItem->SetEnabled(false);
	alwaysOnTopItem->SetEnabled(false);
	SetMenu();
	Pulse();
	};

void TheClockView::initialize() {
	bitmap = new BBitmap(BRect(0,0,Bounds().Width(),Bounds().Height()),B_COLOR_8_BIT,true);
	painter= new TheClockPainter(BRect(0,0,Bounds().Width(),Bounds().Height()),this);
	bitmap->AddChild(painter);
	resizing=false;
	SetViewColor(B_TRANSPARENT_32_BIT);
	menu      = new BPopUpMenu("Main",false,false);
	showSecondsItem    = new BMenuItem("Show seconds",new BMessage(SHOW_SECONDS));
	minuteDotItem    = new BMenuItem("Show minute dots",new BMessage(MINUTE_DOTS));
	alwaysOnTopItem    = new BMenuItem("Always on top",new BMessage(ON_TOP));
	digitalClockItem    = new BMenuItem("digital display",new BMessage(DIGITAL));
	doBeepItem    = new BMenuItem("Beep every hour",new BMessage(DO_BEEP));
	castShadowItem    = new BMenuItem("Cast shadows",new BMessage(CAST_SHADOW));
	showBordersItem    = new BMenuItem("Show borders",new BMessage(SHOW_BORDERS));
	configureItem    = new BMenuItem("Configure...",new BMessage(CONFIGURE));
	saveItem    = new BMenuItem("Save settings",new BMessage(SAVE));
	menu->AddItem(showSecondsItem);
	menu->AddItem(minuteDotItem);
	menu->AddItem(alwaysOnTopItem);
	menu->AddItem(doBeepItem);
	menu->AddItem(castShadowItem);
	menu->AddItem(showBordersItem);
	menu->AddItem(digitalClockItem);
	menu->AddItem(configureItem);
	menu->AddItem(saveItem);
	
	sid = create_sem(1,"clock_sem");
};

TheClockView::~TheClockView()
{
	bitmap->Lock();
	bitmap->RemoveChild(painter);
	delete painter;
	delete bitmap;
	delete menu;
}

status_t TheClockView::Archive(BMessage* data, bool deep) const {
	BView::Archive(data,deep);
	data->AddString("class","TheClockView");
	data->AddString("add_on",signature);
	return B_OK;
};

TheClockView *TheClockView::Instantiate(BMessage *data) {

    if(validate_instantiation(data, "TheClockView")) 
	    return new TheClockView(data); 
    return NULL; 
}

void TheClockView::Draw(BRect updateRect) {
	if (acquire_sem(sid)==B_NO_ERROR) {
	if (!resizing) {
		DrawBitmapAsync(bitmap,updateRect,updateRect);
		}
	else DrawClock(this,this);
	release_sem(sid);};
};

void TheClockView::MouseDown(BPoint point) {
	BMenuItem *selected;
	ConvertToScreen(&point);
	selected=menu->Go(point);
	if (selected==castShadowItem) { 
		castShadow = ! castShadow;
		castShadowItem->SetMarked(castShadow);}
	else if (selected==showSecondsItem) {
		showSeconds = !showSeconds;
		showSecondsItem->SetMarked(showSeconds);}
	else if (selected==digitalClockItem) {
		digitalClock = !digitalClock;
		digitalClockItem->SetMarked(digitalClock);}
	else if (selected==alwaysOnTopItem) {
		alwaysOnTop = !alwaysOnTop;
		alwaysOnTopItem->SetMarked(alwaysOnTop);
		BWindow* window = Window();
		if (alwaysOnTop) window->SetFeel(B_FLOATING_ALL_WINDOW_FEEL);
		else window->SetFeel(B_NORMAL_WINDOW_FEEL);}
	else if (selected==doBeepItem) {
		doBeep = !doBeep;
		doBeepItem->SetMarked(doBeep);}
	else if (selected==digitalClockItem) {
		digitalClock = !digitalClock;
		digitalClockItem->SetMarked(digitalClock);}
	else if (selected==showBordersItem) {
		showBorders = !showBorders;
		showBordersItem->SetMarked(showBorders);
		BWindow* window = Window();
		if (showBorders) window->SetLook(B_TITLED_WINDOW_LOOK);
		else window->SetLook(B_BORDERED_WINDOW_LOOK);}
	else if (selected==configureItem) {
		TheClockConfig* configWin = new TheClockConfig(BRect(100,100,300,300),this);
		configWin->Show();}
	else if (selected==minuteDotItem) {
		minuteDots = !minuteDots;
		minuteDotItem->SetMarked(minuteDots);}
	else if (selected==saveItem) saveSettings();
	Pulse();
};

bool TheClockView::loadSettings() {
	Preferences prefs("x-vnd.raquet-theclock");
	if (prefs.InitCheck()) return false;
	PreferenceSet set(prefs,"TheClock",true);
	if (set.InitCheck()) return false;
	const void* data;
	ssize_t size;
	uint32 type='    ';
	if(!set.GetData("ShowSeconds",data, size, type))
		memcpy(&showSeconds,data,size);
	else showSeconds=true;
	if(!set.GetData("DigitalClock",data, size, type))
		memcpy(&digitalClock,data,size);
	else digitalClock=false;
	if(!set.GetData("AlwaysOnTop",data, size, type))
		memcpy(&alwaysOnTop,data,size);
	else alwaysOnTop=false;
	if(!set.GetData("CastShadow",data, size, type))
		memcpy(&castShadow,data,size);
	else castShadow=true;
	if(!set.GetData("Beep",data, size, type))
		memcpy(&doBeep,data,size);
	else doBeep=false;
	if(!set.GetData("ShowBorders",data, size, type))
		memcpy(&showBorders,data,size);
	else showBorders=true;
	if(!set.GetData("MinuteDots",data, size, type))
		memcpy(&minuteDots,data,size);
	else minuteDots=false;
	if(!set.GetData("BGColor",data, size, type))
		memcpy(&bgColor,data,size);
	else bgColor=BeBackgroundGrey;
	if(!set.GetData("FGColor",data, size, type))
		memcpy(&fgColor,data,size);
	else fgColor=Black;
	if(!set.GetData("ShadowColor",data, size, type))
		memcpy(&shadowColor,data,size);
	else shadowColor=BeShadow;
	if(!set.GetData("SecondColor",data, size, type))
		memcpy(&secColor,data,size);
	else secColor=Red;
	if(!set.GetData("BGType",data, size, type))
		memcpy(&bgType,data,size);
	else bgType=BG_NONE;
	if(!set.GetData("BGFilename",data, size, type)){
		memcpy(&bgFilename,data,size);
    	BFile file;
    	BEntry entry(bgFilename);
    	file.SetTo(&entry,B_READ_WRITE);
    	BBitmap* bm=BTranslationUtils::GetBitmap(&file);
    	if (bm!=NULL) SetBackImage(bm,bgFilename);
		}
	else {
		bgFilename[0]=0;
		bgBitmap=NULL;};
	if (!set.GetData("Font",data, size, type)){
		memcpy(&timeFont,data,size);
		SetFont(&timeFont);
		SetFontSize(timeFont.Size()*0.75);
		GetFont(&secondFont);}		
	else {
		GetFont(&timeFont);
		SetFontSize(timeFont.Size()*0.75);
		GetFont(&secondFont);};		
	BRect frame(100,100,200,200);
	if (!set.GetData("Bounds",data,size,type)) 
			memcpy(&frame,data,sizeof(frame));
		
	if (ownWindow) {
		Window()->Lock();
		Window()->MoveTo(frame.LeftTop());
		Window()->ResizeTo(frame.Width(),frame.Height());
		Window()->Unlock();
		if (alwaysOnTop) Window()->SetFeel(B_FLOATING_ALL_WINDOW_FEEL);
		else Window()->SetFeel(B_NORMAL_WINDOW_FEEL);
		if (showBorders) Window()->SetLook(B_TITLED_WINDOW_LOOK);
		else Window()->SetLook(B_BORDERED_WINDOW_LOOK);
	};
	return true;
};	

void TheClockView::saveSettings() {
	Preferences prefs("x-vnd.raquet-theclock");
	PreferenceSet set(prefs,"TheClock",true);
	set.SetData("ShowSeconds",&showSeconds, sizeof(showSeconds));
	set.SetData("DigitalClock",&digitalClock, sizeof(digitalClock));
	set.SetData("AlwaysOnTop",&alwaysOnTop, sizeof(alwaysOnTop));
	set.SetData("CastShadow",&castShadow, sizeof(castShadow));
	set.SetData("Beep",&doBeep, sizeof(doBeep));
	set.SetData("ShowBorders",&showBorders, sizeof(showBorders));
	set.SetData("MinuteDots",&minuteDots, sizeof(minuteDots));
	set.SetData("BGColor",&bgColor, sizeof(bgColor));
	set.SetData("FGColor",&fgColor, sizeof(fgColor));
	set.SetData("ShadowColor",&shadowColor, sizeof(shadowColor));
	set.SetData("SecondColor",&secColor, sizeof(secColor));
	set.SetData("BGType",&bgType, sizeof(bgType));
	set.SetData("BGFilename",&bgFilename, sizeof(bgFilename));
	set.SetData("Font",&timeFont, sizeof(timeFont));
	if (ownWindow)
		set.SetData("Bounds",&Window()->Frame(),sizeof(Window()->Frame()));
	set.Save();
	};

void TheClockView::SetMenu() {
	showSecondsItem->SetMarked(showSeconds);
	digitalClockItem->SetMarked(digitalClock);
	alwaysOnTopItem->SetMarked(alwaysOnTop);
	doBeepItem->SetMarked(doBeep);
	castShadowItem->SetMarked(castShadow);
	showBordersItem->SetMarked(showBorders);
	minuteDotItem->SetMarked(minuteDots);
}

void TheClockView::Pulse(void){
	BWindow *win = Window();
	if (win) win->Lock();
	time_t now;
	now=time(&now);
	timeRecord=localtime(&now);
	if (resizing)
		resizing=false;
	painter->FillBitmap();
	Invalidate();
	if (win) win->Unlock();
 };
 
void TheClockView::FrameResized(float width, float height) {
	resizing=true;
	BView::FrameResized(width,height);
};

void TheClockView::SetBackImage(BBitmap *bitmap, char*fname) {
	bgBitmap=bitmap;
	strcpy(bgFilename,fname);
};

TheClockPainter::TheClockPainter( BRect frame, TheClockView* p)
	: BView( frame,"TheClockPaintView",0,0)
{
	clockView=p;
};

TheClockPainter::~TheClockPainter() {};

void TheClockPainter::FillBitmap( ) {
	if (acquire_sem(clockView->sid) == B_NO_ERROR) {
	clockView->bitmap->Lock();	
	Bounds().Height();	
	if (Bounds().Width()!=clockView->Bounds().Width() ||
		Bounds().Height()!=clockView->Bounds().Height()) {
		clockView->bitmap->RemoveChild(this);
		delete clockView->bitmap;
		clockView->bitmap = new BBitmap(BRect(0,0,clockView->Bounds().Width(),clockView->Bounds().Height()),B_RGB_32_BIT,true);
		ResizeTo(clockView->Bounds().Width(),clockView->Bounds().Height());
		clockView->bitmap->Lock();
		clockView->bitmap->AddChild(this);
		};
	DrawClock(this,clockView);
	Sync();	
	clockView->bitmap->Unlock();
	release_sem(clockView->sid);};
};

int main(void) {
	new TheClock();
	return(0);
};

TheClock::TheClock() 
	:BApplication(signature){
	win=new TheClockWindow();
	Run();
	};	

void TheClockView::MessageReceived(BMessage *message) {
	uint32 type;
	int32 count;
	entry_ref ref;
	if (message->what==B_REFS_RECEIVED) {
		message->GetInfo("refs",&type,&count);
		if ( type != B_REF_TYPE ) 
    	       return; 
    	if (message->FindRef("refs",count-1,&ref)==B_OK) {
    		BFile file;
    		file.SetTo(&ref,B_READ_WRITE);
    		BEntry entry(&ref);
    		BBitmap* bm=BTranslationUtils::GetBitmap(&file);
    		BPath path;
    		entry.GetPath(&path);
    		if (bm!=NULL) SetBackImage(bm,(char *) path.Path());}}
	else if (message->what=='!FNT') {
		BFont font;
		FindMessageFont(message, "font", 0, &font);
		font_style fontStName;
		font_family fontFam;
		font.GetFamilyAndStyle(&fontFam,&fontStName);
		timeFont.SetFamilyAndStyle(fontFam,fontStName);
		secondFont.SetFamilyAndStyle(fontFam,fontStName);
		int size=font.Size();
		timeFont.SetSize(size);
		secondFont.SetSize(size*0.75);
		Pulse();
		}
	else BView::MessageReceived(message);
};	

TheClockWindow::TheClockWindow()
	: BWindow(BRect(20,20,100,100),"TheClock",B_TITLED_WINDOW,B_WILL_DRAW)
{
	clockView = new TheClockView(BRect(0,0,80,80),"TheClockView",B_FOLLOW_ALL_SIDES,B_WILL_DRAW+B_PULSE_NEEDED+B_FULL_UPDATE_ON_RESIZE+B_FRAME_EVENTS);
	AddChild(clockView);
	clockView->ownWindow=true;
	clockView->loadSettings();
	clockView->Pulse();
	clockView->SetMenu();
	SetPulseRate(1000000);
	BRect frame;
	frame=clockView->Bounds();
	frame.bottom=frame.top+7;
	frame.right=frame.left+7;
	dragger=new BDragger(frame,clockView,0);
	clockView->AddChild(dragger);
	Show();
}

TheClockWindow::~TheClockWindow() {
	RemoveChild(dragger);
	RemoveChild(clockView);
	delete dragger;
	delete clockView;
	};
	
bool TheClockWindow::QuitRequested() {
	be_app->PostMessage(B_QUIT_REQUESTED);
	return true;
	};

TheClockConfig::TheClockConfig(BRect frame, TheClockView* clock)
	:MWindow(frame,"TheClock Settings",B_TITLED_WINDOW,B_WILL_DRAW) {
	clockView=clock;
	colorType=new MPopup("Color of",new BMessage(COLOR_TYPE),this,"Foreground","Background","Shadows","Second Pointer",NULL);
	colorType->Menu()->ItemAt(0)->SetMarked(true);
	colorChooser=new BColorControl(BPoint(0,0),B_CELLS_32x8,4,"BCC",new BMessage(COLOR_CHANGE));
	colorChooser->SetValue(clock->fgColor);
	MBViewWrapper* colorSelect=new MBViewWrapper(colorChooser);
	VGroup* colorGroup= new VGroup(colorType, colorSelect, NULL);
	bgType = new MPopup("Type",new BMessage(BG_TYPE),this,"Off","Tiled","Zoomed",NULL);
	switch (clock->bgType) {
	case BG_NONE:	bgType->Menu()->ItemAt(0)->SetMarked(true);
					break;
	case BG_PATTERN:bgType->Menu()->ItemAt(1)->SetMarked(true);
					break;
	case BG_PICTURE:bgType->Menu()->ItemAt(2)->SetMarked(true);
					break;
					};
	MButton* selectButton=new MButton("Select Image",new BMessage(SELECT_IMAGE));
	HGroup* backgroundGroup=new HGroup(bgType, selectButton,NULL);
	int32	fontNum = count_font_families();
	font_family	fontFamily;
	font_family startFamily;
	font_style startStyle;
	clock->timeFont.GetFamilyAndStyle(&startFamily,&startStyle);
	get_font_family(0,&fontFamily);
	MPopup* fontName = new MPopup ("Name",new BMessage(FONT_FAMILY),this,NULL);
	for(int i=0; i<fontNum; i++){
		get_font_family(i,&fontFamily);
		BMenuItem* familyItem = new BMenuItem(fontFamily, new BMessage(FONT_FAMILY));
		fontName->Menu()->AddItem(familyItem);
		if (strcmp(startFamily,fontFamily)==0) familyItem->SetMarked(true);};
	font_style fontStyleName;
	get_font_style(startFamily,0,&fontStyleName);
	fontStyle = new MPopup ("Style",new BMessage(FONT_STYLE),this,NULL);
	fontNum = count_font_styles(startFamily);
	for(int i=0; i<fontNum; i++){
		get_font_style(startFamily,i,&fontStyleName);
		BMenuItem* styleItem = new BMenuItem(fontStyleName, new BMessage(FONT_STYLE));
		fontStyle->Menu()->AddItem(styleItem);
		if (strcmp(fontStyleName,startStyle)==0) styleItem->SetMarked(true);};
	MPopup* fontSize = new MPopup ("Size",new BMessage(FONT_SIZE),this,"9","10","11","12","14","16","18","20","24","28","32","40","48","60","72","88","104","120",NULL);
	for(int i=0; i<fontSize->Menu()->CountItems(); i++) {
		BMenuItem* item=fontSize->Menu()->ItemAt(i);
		int size;
		sscanf(item->Label(),"%d",&size);
		if (size==clock->timeFont.Size()) item->SetMarked(true);}; 
	HGroup* fontGroup = new HGroup(fontName, fontStyle, fontSize,NULL);
	VGroup* mainGroup = new VGroup(	new MBorder(M_LABELED_BORDER,3,"Colors",colorGroup),
									new MBorder(M_LABELED_BORDER,3,"Pattern",backgroundGroup),
									new MBorder(M_LABELED_BORDER,3,"Font", fontGroup), NULL);
	AddChild(mainGroup);
	};
	
void TheClockConfig::MessageReceived(BMessage* msg) {
	BMenuItem* item;
	switch (msg->what) {
	case SELECT_IMAGE: 
			BFilePanel* panel=new BFilePanel();
			panel->SetTarget(BMessenger(clockView));
			panel->Show();
			break;
	case FONT_FAMILY:
			msg->FindPointer("source",&item);
			font_family fontFamily;
			strcpy(fontFamily,item->Label());
			clockView->timeFont.SetFamilyAndStyle(fontFamily,NULL);
			clockView->secondFont.SetFamilyAndStyle(fontFamily,NULL);
			int fontNum = count_font_styles(fontFamily);
			font_style fontStyleName;
			get_font_style(fontFamily,0,&fontStyleName);
			BMenu* menu = fontStyle->Menu();
			int max=menu->CountItems();
			font_style oldStyle;
			strcpy(oldStyle,menu->FindMarked()->Label());
			for(int32 i=0; i<max; i++) {
				BMenuItem *tmp=menu->ItemAt(0l);
				menu->RemoveItem(0l);
				delete tmp;};
			for(int i=0; i<fontNum; i++){
				get_font_style(fontFamily,i,&fontStyleName);
				BMenuItem* styleItem = new BMenuItem(fontStyleName, new BMessage(FONT_STYLE));
				menu->AddItem(styleItem);
				if (strcmp(oldStyle,fontStyleName)==0) styleItem->SetMarked(true);
				if (i==0) styleItem->SetMarked(true);};
			get_font_style(fontFamily,0,&fontStyleName);
			clockView->timeFont.SetFamilyAndStyle(fontFamily,fontStyleName);
			clockView->secondFont.SetFamilyAndStyle(fontFamily,fontStyleName);
			break;	
	case FONT_STYLE:
			msg->FindPointer("source",&item);
			font_style fontStName;
			font_family fontFam;
			clockView->timeFont.GetFamilyAndStyle(&fontFam,&fontStName);
			strcpy(fontStName,item->Label());
			clockView->timeFont.SetFamilyAndStyle(fontFam,fontStName);
			clockView->secondFont.SetFamilyAndStyle(fontFam,fontStName);
			break;
	case FONT_SIZE:
			msg->FindPointer("source",&item);
			int size;
			sscanf(item->Label(),"%d",&size);
			clockView->timeFont.SetSize(size);
			clockView->secondFont.SetSize(size*0.75);
			break;	
	case COLOR_CHANGE:
			rgb_color color=colorChooser->ValueAsColor();
			switch (colorType->Menu()->IndexOf(colorType->Menu()->FindMarked())) {
			case 0: clockView->fgColor=color;
					break;
			case 1: clockView->bgColor=color;
					break;
			case 2: clockView->shadowColor=color;
					break;
			case 3: clockView->secColor=color;
					break;
				};
			break;
	case COLOR_TYPE:	
			msg->FindPointer("source",&item);
			switch (colorType->Menu()->IndexOf(item)) {
			case 0: colorChooser->SetValue(clockView->fgColor);
					break;
			case 1: colorChooser->SetValue(clockView->bgColor);
					break;
			case 2: colorChooser->SetValue(clockView->shadowColor);
					break;
			case 3: colorChooser->SetValue(clockView->secColor);
					break;
				};
	case BG_TYPE:
			msg->FindPointer("source",&item);
			switch(bgType->Menu()->IndexOf(item)) {
			case 0:	clockView->bgType = BG_NONE;
					break;
			case 1:	clockView->bgType = BG_PATTERN;
					break;
			case 2:	clockView->bgType = BG_PICTURE;
					break;
			};	 		
			break;
	default: BWindow::MessageReceived(msg);	};
	clockView->Pulse();
	};	
	
TheClockConfig::~TheClockConfig() {};	
